BasicGame = {

	logicalWidth: 800,
	logicalHeight: 600,
	debugMode: true,
	soundEnabled: true,
	musicEnabled: true,
	difficulty: 2,//Difficulty goes from 1(easy) to 3(hard), default is 2
	scrollSpeed: 4, //The scrollspeed for the entire game, defined here to always be available
	scrollSpeedEasy: 2,
	scrollSpeedNormal: 4,
	scrollSpeedHard: 6,
	gravity: 200,
	gameOn: false,//Whether the game is running or in a menu etc.
  /* Here we've just got some global level vars that persist regardless of State swaps */
  score: 0,

  /* If the music in your game needs to play through-out a few State swaps, then you could reference it here */
  music: null,

  /* Your game can check BasicGame.orientated in internal loops to know if it should pause or not */
  orientated: false,
	//Stat tracking
	repetitions: 0,
	correctGuesses: 0,
	wrongGuesses: 0,
	kongregate: null

};



BasicGame.Boot = function (game) {
};

BasicGame.Boot.prototype = {

    init: function () {

        this.input.maxPointers = 1;
        this.stage.disableVisibilityChange = true;
				this.game.time.advancedTiming = true;
    		this.game.time.desiredFps = 60;

        if (this.game.device.desktop)
        {
            this.scale.scaleMode = Phaser.ScaleManager.SHOW_ALL;
            this.scale.setMinMax(BasicGame.logicalWidth/2, BasicGame.logicalHeight/2, BasicGame.logicalWidth, BasicGame.logicalHeight);
            this.scale.pageAlignHorizontally = true;
            this.scale.pageAlignVertically = true;
        }
        else
        {
            this.scale.scaleMode = Phaser.ScaleManager.SHOW_ALL;
            this.scale.setMinMax(BasicGame.logicalWidth/8, BasicGame.logicalHeight/8, BasicGame.logicalWidth, BasicGame.logicalHeight);
            this.scale.pageAlignHorizontally = true;
            this.scale.pageAlignVertically = true;
            this.scale.forceOrientation(false, false);
            this.scale.setResizeCallback(this.gameResized, this);
            this.scale.enterIncorrectOrientation.add(this.enterIncorrectOrientation, this);
            this.scale.leaveIncorrectOrientation.add(this.leaveIncorrectOrientation, this);
        }


    },

    preload: function () {
        //  Here we load the assets required for our preloader (in this case a background and a loading bar)
        this.load.image('preloaderBackground', 'images/preloader_bg.jpg');
        this.load.image('preloaderBar', 'images/preloader_bar.jpg');

    },

    create: function () {
        this.state.start('Preloader');

    },

    gameResized: function (width, height) {

        //  This could be handy if you need to do any extra processing if the game resizes.
        //  A resize could happen if for example swapping orientation on a device or resizing the browser window.
        //  Note that this callback is only really useful if you use a ScaleMode of RESIZE and place it inside your main game state.

    },

    enterIncorrectOrientation: function () {

        BasicGame.orientated = false;

        document.getElementById('orientation').style.display = 'block';

    },

    leaveIncorrectOrientation: function () {

        BasicGame.orientated = true;

        document.getElementById('orientation').style.display = 'none';

    }


};
