
BasicGame.Game = function (game) {

	//	When a State is added to Phaser it automatically has the following properties set on it, even if they already exist:

    this.game;		//	a reference to the currently running game
    this.add;		//	used to add sprites, text, groups, etc
    this.camera;	//	a reference to the game camera
    this.cache;		//	the game cache
    this.input;		//	the global input manager (you can access this.input.keyboard, this.input.mouse, as well from it)
    this.load;		//	for preloading assets
    this.math;		//	lots of useful common math operations
    this.sound;		//	the sound manager - add a sound, play one, set-up markers, etc
    this.stage;		//	the game stage
    this.time;		//	the clock
    this.tweens;	//	the tween manager
    this.world;		//	the game world
    this.particles;	//	the particle manager
    this.physics;	//	the physics manager
    this.rnd;		//	the repeatable random number generator

    //	You can use any of these from any function within this State.
    //	But do consider them as being 'reserved words', i.e. don't create a property for your own game called "world" or you'll over-write the world reference.

};

BasicGame.Game.prototype = {

	create: function () {
    var self = this;

    BasicGame.score = 0;
    //Variables
    this.spawnEnemies = false;//Should enemies be spawned?
    this.leftInWave = 0;//Enemies left in the current wave
    this.currentWave = 0;//The current wave
    this.enemiesPerWave = 10;
    this.itemsPerWave = 2;//How many new items should be added to each wave
    this.waveArray = []; //Stores all glyphs to be shown in the current wave
    this.currentGlyphs = {"correct":null, "false":null, "question":null};//Stores the currently displayed glyph
    this.currentGlyphBoxes = {"correct":null, "false":null, "question":null};//Stores the currently displayed GlyphBoxes
    this.HUD = {};//The HUD object containing the score and health data
    this.enemy = null;//The enemy currently on screen
    this.health = 3;//Health of the player
    this.glyphBoxHeight = this.game.height/2;//The height (that is, vertical position) all GlyphBoxes should spawn at
    this.glyphBoxSize = 128;//The size of the glyphBoxes
    this.shuriken = null;//The shuriken on the screen
    this.enemyShuriken = null;//The shuriken thrown by the enemy
    this.toNextSpawn = 0;//Counts down until the next enemy is spawned
    this.inControl = false;

    //Connect to Kongregate API
    console.log("Loading Kongregate API");
    kongregateAPI.loadAPI(this.onComplete);



    //Set up sound
    this.fx = this.add.audio('soundSprites');
    this.fx.allowMultiple = true;
    this.fx.addMarker('hurt',0,0.1);
    this.fx.addMarker('hit',0.1,0.178);
    this.fx.addMarker('newWave',0.28,0.374);

    //Text styles
    this.gameMessageStyle = { font: "48px Press Start 2P", fill: "#ffffff", align: "center" };
    this.scoreMessageStyle = { font: "24px Press Start 2P", fill: "#ffffff", align: "center" };
    this.titleStyle = { font: "64px Press Start 2P", fill: "#ffffff", align: "center" };
    this.HUDStyle = { font: "28px Press Start 2P", fill: "#ffffff", align: "left" };




    //Initial settings
    this.stage.backgroundColor = "#3cbcfc";

    //Create clouds
    this.grpClouds = this.game.add.group();
    for(var i=0; i<3; i++){
      var c = this.grpClouds.create(getRandomInt(0,this.game.width),getRandomInt(0,this.game.height/2),'cloud');
      this.game.physics.arcade.enable(c);
      c.body.velocity.x = -getRandomInt(30,90);
      console.log("spd: "+c.body.velocity.x);
    }


    //Create hills
    this.hills2 = this.add.tileSprite(0,this.game.height-94,this.game.width,140,'hills2');
    this.hills = this.add.tileSprite(0,this.game.height-64,this.game.width,84,'hills');
    this.hills.anchor.set(0,1);
    this.hills2.anchor.set(0,1);

    //Create ground
    this.ground = this.add.tileSprite(0,this.game.height-64,this.game.width,64,'ground');
    this.game.physics.arcade.enable(this.ground);
    this.ground.body.immovable = true;
    this.ground.body.allowGravity = false;

    /*Create title screen objects*/
    //Create a group to hold all titlescreen-related objects
    this.grpTitle = this.game.add.group();

    //Create titlebox
    //this.titleBox = this.add.sprite(this.game.width/2,this.game.height/4,'titleBox');
    //this.titleBox.anchor.set(0.5,0.5);
    this.titleBox = this.add.text(this.game.width/2,this.game.height/4,"KANA RUNNER",this.titleStyle);
    this.titleBox.setShadow(0,8,'#e40058',0);
    this.titleBox.anchor.set(0.5);
    this.grpTitle.add(this.titleBox);

    //Create play button
    this.btn_play = this.add.sprite(this.game.width/2,this.game.height/2,'btn_play');
    this.btn_play.anchor.set(0.5);
    this.btn_play.inputEnabled = true;
    this.btn_play.events.onInputDown.add(this.startGame,self);
    this.grpTitle.add(this.btn_play);

    //Create difficulty settings
    this.difficultyText = this.add.text(this.game.width/2,this.btn_play.y+Math.round(this.game.height-this.btn_play.y)/2,"NORMAL",this.HUDStyle);
    this.difficultyText.anchor.set(0.5);
    this.difficultyText.inputEnabled = true;
    this.difficultyText.events.onInputDown.add(this.increaseDifficulty,this);
    this.updateDifficultyString();
    var arl = this.add.sprite(this.difficultyText.x-this.difficultyText.width/2-8,this.difficultyText.y,'arrow');
    arl.scale.x = -1;
    arl.anchor.y = 0.6;
    var arr = this.add.sprite(this.difficultyText.x+this.difficultyText.width/2+8,this.difficultyText.y,'arrow');
    arr.anchor.y = 0.6;
    this.grpTitle.add(this.difficultyText);
    this.grpTitle.add(arl);
    this.grpTitle.add(arr);


    //Create sound button
    this.btn_sound = this.add.sprite(8,this.game.height-72,'soundIcon');
    this.btn_sound.inputEnabled = true;
    this.btn_sound.events.onInputDown.add(this.toggleSound,self);
    this.grpTitle.add(this.btn_sound);

    //Add a general-use message box
    this.gameMessage = this.game.add.text(this.game.width/2,this.game.height/4,"MESSAGE",this.gameMessageStyle);
    this.gameMessage.anchor.set(0.5);
    this.gameMessage.visible = false;

    //Load kana data
    this.glyphArray = deepCopy(kanadata);
    this.glyphArray = reverseSort(this.glyphArray);




	},

	update: function () {
    //Scroll the ground
    this.hills2.tilePosition.x -= BasicGame.scrollSpeed/4;
    this.hills.tilePosition.x -= BasicGame.scrollSpeed/2;
    this.ground.tilePosition.x -= BasicGame.scrollSpeed;

    //Wrap the clouds
    this.grpClouds.forEach(function(c){
      if(c.x < -c.width){
        c.x = c.game.width;
        c.y = getRandomInt(0,c.game.height/2);
        c.body.velocity.x = -getRandomInt(30,90);
      }
    });

    //Collision detection
    this.game.physics.arcade.collide(this.player,this.ground);
    this.game.physics.arcade.overlap(this.player,this.enemy,this.hurt,null,this);
    this.game.physics.arcade.collide(this.shuriken,this.enemy,this.enemyHit,null,this);
    this.game.physics.arcade.collide(this.player,this.enemyShuriken,function(){this.enemyShuriken.destroy();this.hurt();},null,this);

    //Spawning of enemies
    if(this.spawnEnemies){
      if(this.toNextSpawn > 0){
        this.toNextSpawn--;
        if(this.toNextSpawn == 0){
          this.spawnEnemy();
        }
      }
    }

    //If enemy is too close, count it as a wrong guess
    if(this.inControl && this.currentGlyphBoxes.question != null){
      if(this.currentGlyphBoxes.question.x - this.player.x <= this.glyphBoxSize*1.5){
        this.handleWrongGuess();
      }
    }


	},

	quitGame: function (pointer) {

		//	Here you should destroy anything you no longer need.
		//	Stop music, delete sprites, purge caches, free resources, all that good stuff.

		//	Then let's go back to the main menu.
		this.state.start('MainMenu');

	},

  /*Create a player and start the main game loop*/
  startGame: function(){
    //Create HUD
    this.createHUD();
    //Remove title screen objects
    this.grpTitle.destroy(true,false);
    //Create player
    this.createPlayer();

    //Set current wave as 1
    this.currentWave = 1;

    //Show wave message
    this.showWaveMessage();

  },

  showWaveMessage: function(){//TODO: Rename and/or split this function
    this.grpWaveMessage = this.game.add.group();
    //Announce the new wave on screen
    this.setMessage("WAVE "+this.currentWave);
    if(BasicGame.soundEnabled){
      this.playSound('newWave');
    }
    //Create a "Start Wave" button
    var btn_wave = this.game.add.sprite(this.game.width/2,this.game.height/2+32,'btn_play');
    btn_wave.anchor.set(0.5);
    btn_wave.inputEnabled = true;
    btn_wave.events.onInputDown.add(this.startWave,this);
    //Get itemsPerWave glyphs or however many remain
    var t = Math.min(this.itemsPerWave,this.glyphArray.length);
    for(var i=0; i< t; i++){
      //Move the next glyph from the full array to the current wave array; skip if there are no more glyphs
      if(this.glyphArray.length > 0){
        this.waveArray.push(this.glyphArray.pop());

        //Calculate the position for the GlyphBoxes
        var pos_x = Math.round((this.game.width / this.itemsPerWave)/2+i*(this.game.width / this.itemsPerWave));
        //Grab the newest glyph in the waveArray
        var g = this.waveArray[this.waveArray.length-1];
        var newQ = new GlyphBox(this,pos_x,this.game.height/2,g.glyphQ);
        var newA = new GlyphBox(this,pos_x,this.game.height/2+128,g.glyphA);
        this.grpWaveMessage.add(newQ);
        this.grpWaveMessage.add(newA);
      }
    }
    //this.grpWaveMessage.add(waveMsg);
    this.grpWaveMessage.add(btn_wave);
  },

  //This function starts a wave and enables spawning of enemies
  startWave: function(){
    this.grpWaveMessage.destroy();
    this.hideMessage();
    this.spawnEnemies = true;
    this.leftInWave = this.enemiesPerWave;
    this.resetSpawnTimer();
  },

  createPlayer: function(){
    this.player = new Player(this.game,this.game.width/6,this.ground.y-64);
    //DEBUG STUFF
    //this.player.inputEnabled = true;
    //this.player.events.onInputDown.add(function(){this.game.state.start(this.game.state.current);},this);

  },

  createHUD: function(){
    //Define a style for the score HUD element
    var score = this.game.add.text(0,0,"SCORE: 0",this.HUDStyle);
    this.HUD.score = score;
    //Add health bar
    this.HUD.health = new Healthbar(this,3,'heart',60);
    //Position the bar in the upper-right corner
    this.HUD.health.x = this.game.width - this.HUD.health.width;
  },

  //Returns a glyph for the current wave. If prev != null, return a random glyph that is different from prev
  getRandomGlyph: function(prev){
    var g = this.waveArray[getRandomInt(0,this.waveArray.length)];//Grab a random viable glyph
    if(prev != null){
      while(prev == g.glyphA){
        g = this.waveArray[getRandomInt(0,this.waveArray.length)];
      }
    }
    debugLog("Next Glyph: "+g.glyphQ);
    return g;
  },

  //Get and store the current glyph, that is, the glyph that will be spawned. The question, answer and wrong answer glyphs are stored in currentGlyphs.
  setCurrentGlyph: function(){
    var g = this.getRandomGlyph();//Grab the glyph to be used
    this.currentGlyphs.question = g.glyphQ;//Store the question glyph
    this.currentGlyphs.correct = g.glyphA;//Store the answer glyph
    var fg = this.getRandomGlyph(this.currentGlyphs.correct);//Grab a false answer glyph...
    this.currentGlyphs.false = fg.glyphA;//...and store it
  },

  //Creates the answer glyphbox above the player
  createAnswerGlyphs: function(){//TODO Rename to createAnswerGlyphBoxes
    var shift = tossCoin()?1:-1;//Randomize position of correct answer
    //Show correct answer
    var c = new GlyphBox(this,this.player.x+(64*shift),this.glyphBoxHeight,this.currentGlyphs.correct,true);
    c.events.onInputDown.add(this.handleCorrectGuess,this);
    this.currentGlyphBoxes.correct = c;
    debugLog("Correct glyph: "+this.currentGlyphs.correct);
    //Show wrong answer
    var f = new GlyphBox(this,this.player.x-(64*shift),this.glyphBoxHeight,this.currentGlyphs.false,true);
    f.events.onInputDown.add(this.handleWrongGuess,this);
    this.currentGlyphBoxes.false = f;
    //TODO: Add these Glyphboxes to a state-wide group so they can be selected and destroyed
  },

  //Spawn an enemy, set the current glyph and display all Glyphboxes.
  spawnEnemy: function(){
    debugLog("Spawning enemy");
    this.slowDownTime();
    this.setCurrentGlyph();
    this.createAnswerGlyphs();
    //Spawn enemy
    this.enemy = new Enemy(this.game,this.game.width+this.glyphBoxSize,this.ground.y);
    //Spawn question glyph box
    this.currentGlyphBoxes.question = new GlyphBox(this,this.enemy.x,this.glyphBoxHeight,this.currentGlyphs.question,false);
    this.currentGlyphBoxes.question.setVel(this.enemy.body.velocity.x);
    this.inControl = true;
    BasicGame.repetitions++;

  },

  handleCorrectGuess: function(){
    debugLog("Correct!");
    this.inControl = false;
    this.leftInWave--;
    this.currentGlyphBoxes.false.destroy();
    this.currentGlyphBoxes.correct.inputEnabled = false;
    //Spawn shuriken
    this.shuriken = new Shuriken(this,this.player.x,this.player.y+this.player.height/2);
    this.restoreTime();
    BasicGame.correctGuesses++;
  },

  handleWrongGuess: function(){
    debugLog("Wrong!");
    this.inControl = false;
    //TODO: Vibrate phone
    this.enemyShuriken = new Shuriken(this,this.enemy.x,this.enemy.y-this.enemy.height/2,-1);
    this.enemy.vanish();
    //Destroy all glyphboxes
    this.currentGlyphBoxes.question.destroy();
    this.currentGlyphBoxes.correct.destroy();
    this.currentGlyphBoxes.false.destroy();
    this.restoreTime();
    BasicGame.wrongGuesses++;
  },

  enemyHit: function(){
    this.enemy.destroy();
    this.shuriken.destroy();
    this.playSound('hit');
    //Increase score
    this.addToScore(10);
    //Destroy the remaining glyphboxes
    this.currentGlyphBoxes.question.destroy();
    this.currentGlyphBoxes.correct.destroy();
    //Schedule the next spawn
    this.resetSpawnTimer();

  },

  //Increase the score and update the display
  addToScore: function(v){
    var s = v*BasicGame.difficulty;
    BasicGame.score += s;
    this.HUD.score.text = "SCORE: "+BasicGame.score;
  },

  hurt: function(){
    debugLog("Hurt");
    this.playSound('hurt');
    this.restoreTime();//Restore normal flow of time
    this.player.flicker();
    this.health--;
    this.HUD.health.setHealth(this.health);//Update the health HUD
    //TODO: If health < 0 then game over
    if(this.health == 0){
      this.endGame();
    }
    this.resetSpawnTimer();
  },

  //Fly an object out the upper side of the screen using tweens
  flyOut: function(obj,dest){
    var d = dest || false;
    this.game.add.tween(obj).to({y:0-obj.height},500,null,true,1000);
  },

  resetSpawnTimer: function(){
    if(this.leftInWave == 0){
      this.prepareNextWave();
    }
    this.toNextSpawn = 2*this.game.time.fps;
  },

  prepareNextWave: function(){
    this.toNextSpawn = 2*this.game.time.fps;
    this.spawnEnemies = false;
    this.currentWave++;
    this.showWaveMessage();
  },

  //Slow down time by a factor of factor
  slowDownTime: function(factor){
    return;
    var f = typeof factor==='number'?factor:3;
    this.game.time.slowMotion = f;
  },

  //Restore time to normal speed
  restoreTime: function(){
    this.game.time.slowMotion = 1.0;
  },

  endGame: function(){
    this.spawnEnemies = false;
    this.player.destroy();
    this.setMessage("Game Over");
    //Display score
    var txt_score = this.game.add.text(this.game.width/2,(this.game.height-this.gameMessage.y)/2,"Final Score:\n"+BasicGame.score,this.scoreMessageStyle);
    txt_score.addColor('#e40058',12);
    txt_score.anchor.set(0.5);

    //Display the replay button
    var btn_restart = this.game.add.sprite(this.game.width/2,this.game.height/2+64,'btn_replay');
    btn_restart.anchor.set(0.5);
    btn_restart.inputEnabled = true;
    btn_restart.events.onInputDown.add(function(){this.game.state.start(this.game.state.current);},this);
    this.submitHighscore();

  },

  setMessage: function(txt){
    this.gameMessage.visible = true;
    this.gameMessage.text = txt;
    this.gameMessage.setShadow(0,4,'rgb(0, 0, 0)',0);
    this.gameMessage.alpha = 1;
  },

  hideMessage: function(){
    this.gameMessage.alpha = 0;
  },

  //Toggle sfx on or off
  toggleSound: function(){
    BasicGame.soundEnabled = !BasicGame.soundEnabled;
    if(BasicGame.soundEnabled){
      this.btn_sound.frame = 0;
    } else{
      this.btn_sound.frame = 1;
    }
  },

  playSound: function(snd){
    if(BasicGame.soundEnabled){
      this.fx.play(snd);
    }

  },

  submitHighscore: function(){
    BasicGame.kongregate.stats.submit("HighScore",BasicGame.score);
    BasicGame.kongregate.stats.submit("Repetitions",BasicGame.repetitions);
    BasicGame.kongregate.stats.submit("Correct",BasicGame.correctGuesses);
    BasicGame.kongregate.stats.submit("Wrong",BasicGame.wrongGuesses);
  },

  onComplete: function(){
    console.log("Kongregate API loaded");
    BasicGame.kongregate = kongregateAPI.getAPI();
  },

  increaseDifficulty: function(){
    if(BasicGame.difficulty <3){
      BasicGame.difficulty++;
    } else{
      BasicGame.difficulty = 1;
    }
    //Adjust the scrollspeed
    switch(BasicGame.difficulty){
      case 1:
        BasicGame.scrollSpeed = BasicGame.scrollSpeedEasy;
        this.difficultyText.text = "EASY";
        break;
      case 2:
        BasicGame.scrollSpeed = BasicGame.scrollSpeedNormal;
        this.difficultyText.text = "NORMAL";
        break;
      case 3:
        BasicGame.scrollSpeed = BasicGame.scrollSpeedHard;
        this.difficultyText.text = "HARD";
        break;
    }
    debugLog("Scrollspeed: "+BasicGame.scrollSpeed);
  },

  updateDifficultyString: function(){
    switch(BasicGame.difficulty){
      case 1:
        this.difficultyText.text = "EASY";
        break;
      case 2:
        this.difficultyText.text = "NORMAL";
        break;
      case 3:
        this.difficultyText.text = "HARD";
        break;
    }
  }



};
